@@ -34,7 +34,8 @@
 #include <errno.h>
 #include <string.h>
 #include <glib/gstdio.h>
-
+#include <glib/gi18n-lib.h>
+#include <libsoup/soup-cookie.h>
 #include <rest/rest-xml-node.h>
 #include <rest/rest-xml-parser.h>
 
@@ -46,6 +47,7 @@ enum {
     PROP_0,
     PROP_CA_CERT,
     PROP_ADMIN,
+    PROP_SESSION_ID
 };
 
 #define CA_CERT_FILENAME "ca.crt"
@@ -138,6 +140,26 @@ static void dump_vm(OvirtVm *vm)
 #endif
 
 
+static RestProxyCall *ovirt_rest_call_new(OvirtProxy *proxy,
+                                          const char *method,
+                                          const char *href)
+{
+    RestProxyCall *call;
+
+    g_return_val_if_fail(OVIRT_IS_PROXY(proxy), NULL);
+
+    call = REST_PROXY_CALL(ovirt_action_rest_call_new(REST_PROXY(proxy)));
+    if (method != NULL) {
+        rest_proxy_call_set_method(call, method);
+    }
+    rest_proxy_call_set_function(call, href);
+    /* FIXME: to set or not to set ?? */
+    rest_proxy_call_add_header(call, "All-Content", "true");
+
+    return call;
+}
+
+
 RestXmlNode *ovirt_proxy_get_collection_xml(OvirtProxy *proxy,
                                             const char *href,
                                             GError **error)
@@ -147,10 +169,7 @@ RestXmlNode *ovirt_proxy_get_collection_xml(OvirtProxy *proxy,
 
     g_return_val_if_fail(OVIRT_IS_PROXY(proxy), NULL);
 
-    call = REST_PROXY_CALL(ovirt_action_rest_call_new(REST_PROXY(proxy)));
-    href = ovirt_utils_strip_api_base_dir(href);
-    rest_proxy_call_set_function(call, href);
-    rest_proxy_call_add_header(call, "All-Content", "true");
+    call = ovirt_rest_call_new(proxy, "GET", href);
 
     if (!rest_proxy_call_sync(call, error)) {
         if ((error != NULL) && (*error != NULL)) {
@@ -215,7 +234,7 @@ call_async_cb(RestProxyCall *call, const GError *error,
         g_simple_async_result_set_from_error(result, error);
     } else {
         GError *call_error = NULL;
-        gboolean callback_result = FALSE;
+        gboolean callback_result = TRUE;
 
         if (data->call_async_cb != NULL) {
             callback_result = data->call_async_cb(data->proxy, call,
@@ -234,27 +253,6 @@ call_async_cb(RestProxyCall *call, const GError *error,
 }
 
 
-OvirtRestCall *ovirt_rest_call_new(OvirtProxy *proxy,
-                                   const char *method,
-                                   const char *href)
-{
-    OvirtRestCall *call;
-
-    g_return_val_if_fail(OVIRT_IS_PROXY(proxy), NULL);
-
-    call = OVIRT_REST_CALL(ovirt_action_rest_call_new(REST_PROXY(proxy)));
-    if (method != NULL) {
-        rest_proxy_call_set_method(REST_PROXY_CALL(call), method);
-    }
-    href = ovirt_utils_strip_api_base_dir(href);
-    rest_proxy_call_set_function(REST_PROXY_CALL(call), href);
-    /* FIXME: to set or not to set ?? */
-    rest_proxy_call_add_header(REST_PROXY_CALL(call), "All-Content", "true");
-
-    return call;
-}
-
-
 void ovirt_rest_call_async(OvirtRestCall *call,
                            GSimpleAsyncResult *result,
                            GCancellable *cancellable,
@@ -263,7 +261,7 @@ void ovirt_rest_call_async(OvirtRestCall *call,
                            GDestroyNotify destroy_func)
 {
     OvirtProxy *proxy;
-    GError *error;
+    GError *error = NULL;
     OvirtProxyCallAsyncData *data;
 
     g_return_if_fail((cancellable == NULL) || G_IS_CANCELLABLE(cancellable));
@@ -287,7 +285,6 @@ void ovirt_rest_call_async(OvirtRestCall *call,
         g_warning("Error while getting collection XML");
         g_simple_async_result_set_from_error(result, error);
         g_simple_async_result_complete(result);
-        g_object_unref(G_OBJECT(call));
         ovirt_proxy_call_async_data_free(data);
     }
 }
@@ -358,7 +355,7 @@ void ovirt_proxy_get_collection_xml_async(OvirtProxy *proxy,
                                           GDestroyNotify destroy_func)
 {
     OvirtProxyGetCollectionAsyncData *data;
-    OvirtRestCall *call;
+    RestProxyCall *call;
 
     data = g_slice_new0(OvirtProxyGetCollectionAsyncData);
     data->parser = callback;
@@ -367,7 +364,7 @@ void ovirt_proxy_get_collection_xml_async(OvirtProxy *proxy,
 
     call = ovirt_rest_call_new(proxy, "GET", href);
 
-    ovirt_rest_call_async(call, result, cancellable,
+    ovirt_rest_call_async(OVIRT_REST_CALL(call), result, cancellable,
                           get_collection_xml_async_cb, data,
                           (GDestroyNotify)ovirt_proxy_get_collection_async_data_destroy);
     g_object_unref(call);
@@ -379,19 +376,10 @@ static GFile *get_ca_cert_file(OvirtProxy *proxy)
     gchar *base_uri = NULL;
     gchar *ca_uri = NULL;
     GFile *ca_file = NULL;
-    gsize suffix_len;
 
     g_object_get(G_OBJECT(proxy), "url-format", &base_uri, NULL);
     if (base_uri == NULL)
         goto error;
-    if (g_str_has_suffix(base_uri, OVIRT_API_BASE_DIR))
-        suffix_len = strlen(OVIRT_API_BASE_DIR);
-    else if (g_str_has_suffix(base_uri, "/api"))
-        suffix_len = strlen("/api");
-    else
-        g_return_val_if_reached(NULL);
-
-    base_uri[strlen(base_uri) - suffix_len] = '\0';
 
     ca_uri = g_build_filename(base_uri, CA_CERT_FILENAME, NULL);
     g_debug("CA certificate URI: %s", ca_uri);
@@ -405,7 +393,7 @@ error:
 
 static GByteArray *get_ca_cert_data(OvirtProxy *proxy)
 {
-    char *ca_file;
+    char *ca_file = NULL;
     char *content;
     gsize length;
     GError *error = NULL;
@@ -508,7 +496,7 @@ static char *write_to_tmp_file(const char *template,
             if ((errno != EINTR) || (errno != EAGAIN)) {
                 g_set_error(error, G_FILE_ERROR,
                             g_file_error_from_errno(errno),
-                            "Failed to write to '%s': %s",
+                            _("Failed to write to '%s': %s"),
                             tmp_file, strerror(errno));
                 goto end;
             }
@@ -526,7 +514,7 @@ end:
         if (close_status != 0) {
             g_set_error(error, G_FILE_ERROR,
                         g_file_error_from_errno(errno),
-                        "Failed to close '%s': %s",
+                        _("Failed to close '%s': %s"),
                         result, strerror(errno));
             g_free(result);
             result = NULL;
@@ -550,15 +538,21 @@ static gboolean set_ca_cert_from_data(OvirtProxy *proxy,
      * the ca-cert property, we need to create a temporary file in order
      * to be able to keep the ssl-ca-file property synced with it
      */
-    char *ca_file_path;
+    char *ca_file_path = NULL;
     GError *error = NULL;
     gboolean result = FALSE;
 
-    ca_file_path = write_to_tmp_file("govirt-ca-XXXXXX.crt", ca_cert_data, ca_cert_len, &error);
-    if (ca_file_path == NULL) {
-        g_warning("Failed to create temporary file for CA certificate: %s",
-                  error->message);
-        goto end;
+    if (ca_cert_data != NULL) {
+        ca_file_path = write_to_tmp_file("govirt-ca-XXXXXX.crt",
+                                         ca_cert_data, ca_cert_len,
+                                         &error);
+        if (ca_file_path == NULL) {
+            g_warning("Failed to create temporary file for CA certificate: %s",
+                      error->message);
+            goto end;
+        }
+    } else {
+        ca_file_path = NULL;
     }
 
     ovirt_proxy_set_tmp_ca_file(proxy, ca_file_path);
@@ -585,7 +579,8 @@ gboolean ovirt_proxy_fetch_ca_certificate(OvirtProxy *proxy, GError **error)
 
     source = get_ca_cert_file(proxy);
     if (source == NULL) {
-        g_set_error(error, OVIRT_ERROR, OVIRT_ERROR_BAD_URI, "could not extract ca cert filename from URI");
+        g_set_error(error, OVIRT_ERROR, OVIRT_ERROR_BAD_URI,
+                    _("Could not extract CA certificate filename from URI"));
         goto error;
     }
 
@@ -690,12 +685,40 @@ static void ovirt_proxy_get_property(GObject *object,
     case PROP_ADMIN:
         g_value_set_boolean(value, proxy->priv->admin_mode);
         break;
+    case PROP_SESSION_ID:
+        g_value_set_string(value, proxy->priv->jsessionid);
+        break;
 
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     }
 }
 
+
+static void ovirt_proxy_set_session_id(OvirtProxy *proxy, const char *session_id)
+{
+    char *url;
+    char *domain;
+
+    g_object_get(G_OBJECT(proxy), "url-format", &url, NULL);
+    g_return_if_fail(url != NULL);
+    if (g_str_has_prefix(url, "https://")) {
+        domain = url + strlen("https://");
+    } else {
+        domain = url;
+    }
+
+    g_free(proxy->priv->jsessionid);
+    proxy->priv->jsessionid = g_strdup(session_id);
+    if (proxy->priv->jsessionid != NULL) {
+        SoupCookie *cookie;
+        cookie = soup_cookie_new("JSESSIONID", session_id, domain, "/api", -1);
+        soup_cookie_jar_add_cookie(proxy->priv->cookie_jar, cookie);
+    }
+    g_free(url);
+}
+
+
 static void ovirt_proxy_set_property(GObject *object,
                                      guint prop_id,
                                      const GValue *value,
@@ -707,7 +730,11 @@ static void ovirt_proxy_set_property(GObject *object,
     case PROP_CA_CERT: {
         GByteArray *ca_cert;
         ca_cert = g_value_get_boxed(value);
-        set_ca_cert_from_data(proxy, (char *)ca_cert->data, ca_cert->len);
+        if (ca_cert != NULL) {
+            set_ca_cert_from_data(proxy, (char *)ca_cert->data, ca_cert->len);
+        } else {
+            set_ca_cert_from_data(proxy, NULL, 0);
+        }
         break;
     }
 
@@ -715,6 +742,10 @@ static void ovirt_proxy_set_property(GObject *object,
         proxy->priv->admin_mode = g_value_get_boolean(value);
         break;
 
+    case PROP_SESSION_ID:
+        ovirt_proxy_set_session_id(proxy, g_value_get_string(value));
+        break;
+
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     }
@@ -730,6 +761,11 @@ ovirt_proxy_dispose(GObject *obj)
         proxy->priv->vms = NULL;
     }
 
+    if (proxy->priv->cookie_jar) {
+        g_object_unref(G_OBJECT(proxy->priv->cookie_jar));
+        proxy->priv->cookie_jar = NULL;
+    }
+
     G_OBJECT_CLASS(ovirt_proxy_parent_class)->dispose(obj);
 }
 
@@ -739,6 +775,7 @@ ovirt_proxy_finalize(GObject *obj)
     OvirtProxy *proxy = OVIRT_PROXY(obj);
 
     ovirt_proxy_set_tmp_ca_file(proxy, NULL);
+    g_free(proxy->priv->jsessionid);
 
     G_OBJECT_CLASS(ovirt_proxy_parent_class)->finalize(obj);
 }
@@ -770,6 +807,14 @@ ovirt_proxy_class_init(OvirtProxyClass *klass)
                                                          FALSE,
                                                          G_PARAM_READWRITE |
                                                          G_PARAM_STATIC_STRINGS));
+    g_object_class_install_property(oclass,
+                                    PROP_SESSION_ID,
+                                    g_param_spec_string("session-id",
+                                                        "session-id",
+                                                        "oVirt/RHEV JSESSIONID",
+                                                        NULL,
+                                                        G_PARAM_READWRITE |
+                                                        G_PARAM_STATIC_STRINGS));
 
     g_type_class_add_private(klass, sizeof(OvirtProxyPrivate));
 }
@@ -791,13 +836,65 @@ ovirt_proxy_init(OvirtProxy *self)
     handler_id = g_signal_connect(G_OBJECT(self), "notify::ssl-ca-file",
                                   (GCallback)ssl_ca_file_changed, NULL);
     self->priv->ssl_ca_file_changed_id = handler_id;
+    self->priv->cookie_jar = soup_cookie_jar_new();
+    rest_proxy_add_soup_feature(REST_PROXY(self),
+                                SOUP_SESSION_FEATURE(self->priv->cookie_jar));
 }
 
-OvirtProxy *ovirt_proxy_new(const char *uri)
+/* FIXME : "uri" should just be a base domain, foo.example.com/some/path
+ * govirt will then prepend https://
+ * /api/ is part of what librest call 'function'
+ * -> get rid of all the /api/ stripping here and there
+ */
+OvirtProxy *ovirt_proxy_new(const char *hostname)
 {
-    return g_object_new(OVIRT_TYPE_PROXY,
-                        "url-format", uri,
-                        NULL);
+    char *uri;
+    OvirtProxy *proxy;
+    gsize suffix_len = 0;
+    int i;
+
+    if (!g_str_has_prefix(hostname, "http://") && !g_str_has_prefix(hostname, "https://")) {
+        uri = g_strconcat("https://", hostname, NULL);
+    } else {
+        /* Fallback code for backwards API compat, early libgovirt versions
+         * expected a full fledged URI */
+        g_warning("Passing a full http:// or https:// URI to "
+                  "ovirt_proxy_new() is deprecated");
+        uri = g_strdup(hostname);
+    }
+
+    /* More backwards compat code, strip "/api" from URI */
+    if (g_str_has_suffix(uri, "api")) {
+        suffix_len = strlen("api");
+    } else if (g_str_has_suffix(uri, "/api")) {
+        suffix_len = strlen("/api");
+    } else if (g_str_has_suffix(uri, "/api/")) {
+        suffix_len = strlen("/api/");
+    }
+    if (suffix_len != 0) {
+        g_warning("Passing an URI ending in /api to ovirt_proxy_new() "
+                  "is deprecated");
+        uri[strlen(uri) - suffix_len] = '\0';
+    }
+
+    /* Strip trailing '/' */
+    for (i = strlen(uri)-1; i >= 0; i--) {
+        if (uri[i] != '/') {
+            break;
+        }
+        uri[i] = '\0';
+    }
+
+    /* We disable cookies upon OvirtProxy creation as we will be
+     * adding our own cookie jar to OvirtProxy
+     */
+    proxy =  OVIRT_PROXY(g_object_new(OVIRT_TYPE_PROXY,
+                                      "url-format", uri,
+                                      "disable-cookies", TRUE,
+                                      NULL));
+    g_free(uri);
+
+    return proxy;
 }
 
 
@@ -818,7 +915,7 @@ static void ovirt_proxy_set_api_from_xml(OvirtProxy *proxy,
  * @proxy: a #OvirtProxy
  * @error: #GError to set on error, or NULL
  *
- * Return value: (transfer full):
+ * Return value: (transfer none):
  */
 OvirtApi *ovirt_proxy_fetch_api(OvirtProxy *proxy, GError **error)
 {
@@ -826,7 +923,7 @@ OvirtApi *ovirt_proxy_fetch_api(OvirtProxy *proxy, GError **error)
 
     g_return_val_if_fail(OVIRT_IS_PROXY(proxy), FALSE);
 
-    api_node = ovirt_proxy_get_collection_xml(proxy, "", error);
+    api_node = ovirt_proxy_get_collection_xml(proxy, "/api", error);
     if (api_node == NULL) {
         return NULL;
     }
@@ -869,7 +966,7 @@ void ovirt_proxy_fetch_api_async(OvirtProxy *proxy,
     result = g_simple_async_result_new (G_OBJECT(proxy), callback,
                                         user_data,
                                         ovirt_proxy_fetch_api_async);
-    ovirt_proxy_get_collection_xml_async(proxy, "", result, cancellable,
+    ovirt_proxy_get_collection_xml_async(proxy, "/api", result, cancellable,
                                          fetch_api_async_cb, NULL, NULL);
 }
 
